#ifndef COMMON_H_INCLUDED
#define COMMON_H_INCLUDED

#define min(a,b) (((a)<(b))?(a):(b))
#define max(a,b) (((a)>(b))?(a):(b))
#undef RGB

typedef unsigned char  uchar;
typedef unsigned short ushort;
typedef unsigned int   uint;
typedef uchar Byte;

template<class t>
struct vec2
{
    t x, y;
};

template<class t>
struct vec3
{
    t x, y, z;
};

template<class t>
struct vec4
{
    t x, y, z, w;
};

struct Point
{
        int X;
        int Y;

        Point() {};
        Point(int x, int y)
        {
                this->X = x;
                this->Y = y;
        };
        bool operator != (Point a){ return this->X != a.X || this->Y != a.Y; };
        bool operator == (Point a){ return this->X == a.X && this->Y == a.Y; };
        Point operator + (Point a){ return Point(this->X + a.X, this->Y + a.Y); }
        Point operator + (int a){ return Point(this->X + a, this->Y + a); }
        Point operator - (Point a){ return Point(this->X - a.X, this->Y - a.Y); }
        Point operator - (int a){ return Point(this->X - a, this->Y - a); }
        Point operator * (Point a){ return Point(this->X * a.X, this->Y * a.Y); }
        Point operator * (int a){ return Point(this->X * a, this->Y * a); }
        Point operator / (Point a){ return Point(this->X / a.X, this->Y / a.Y); }
        Point operator / (int a){ return Point(this->X / a, this->Y / a); }

        float Lingrh();
        float Angle();
};

struct Rect
{
        int X;
        int Y;
        int Width;
        int Height;
        Rect() {}
        Rect(int X, int Y, int Width, int Height)
        {
                this->X = X;
                this->Y = Y;
                this->Width = Width;
                this->Height = Height;
        }
};

struct RGBA
{
        Byte b;
        Byte g;
        Byte r;
        Byte a;

        RGBA(uint value = 0)
        {
                a = (Byte)(value >> 24);
                r = (Byte)(value >> 16);
                g = (Byte)(value >> 8);
                b = (Byte)value;
        };
        //
        bool operator != (RGBA &another)
        {
                return this->b != another.b || this->g != another.g || this->r != another.r || this->a != another.a;
        };
        //
        bool operator == (RGBA &another)
        {
                return this->b == another.b && this->g == another.g && this->r == another.r && this->a == another.a;
        };
};


struct RGB
{
        Byte b;
        Byte g;
        Byte r;
        //
        RGB( uint value = 0)
        {
                r = (Byte)(value >> 16);
                g = (Byte)(value >> 8);
                b = (Byte)value;
        };
        //
        bool operator != ( RGB &another )
        {
                return this->b != another.b || this->g != another.g || this->r != another.r;
        };
        //
        bool operator == ( RGB &another )
        {
                return this->b == another.b && this->g == another.g && this->r == another.r;
        };
};

struct Player
{
        Point vect;
        Point position;
        float angle;

        Player() {}
        Player(Point position)
        {
                this->vect = Point(0, -1);
                this->position = position;
                this->angle = 270;
        };
};




#endif // COMMON_H_INCLUDED
